#include <stdio.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/videodev.h>

int channel = 0;
int opt_v = 0;
char device[1024];

void dump_vidcap(struct video_capability* vidcap);
void dump_vidchan(struct video_channel* vidchan);
void dump_vidtuner(struct video_tuner* vidtuner);

void main(int argc, char *argv[])
{
static	const struct option opts[] = {
	 { "verbose", no_argument, &opt_v, 1 },
	 { "channel", required_argument, NULL, 'c' },
	 { 0 }
	};
	int	ch;
	int	fd;
	int	i;
	struct	video_capability vidcap;
	struct	video_channel vidchan;
	struct	video_mmap vidmap;

	strcpy(device,"/dev/video0");

	while ((ch=getopt_long(argc,argv,"c:",opts,NULL))!=-1)
	 switch (ch) {
	  case 0:	/* long option */
			break;
	  case 'd':	/* device */
			strcpy(device, optarg);
			break;
	  case 'c':	/* channel */
			channel = atoi(optarg);
			break;
	  case 'v':	/* verbose */
			opt_v = 1;
			break;
	}

	fd = open(device,O_RDONLY);
	if (fd<0) {
		perror("open");
		exit(1);
	}

	if (ioctl(fd,VIDIOCGCAP,&vidcap) < 0) {
		perror("ioctl(VIDIOCGCAP)");
		exit(1);
	}

	if (opt_v) {
		dump_vidcap(&vidcap);
		for (i=0; i<vidcap.channels; i++) {
			vidchan.channel = i;
			if (ioctl(fd,VIDIOCGCHAN,&vidchan) < 0) {
				perror("ioctl(VIDIOCGCHAN)");
				exit(1);
			}
			if (opt_v) dump_vidchan(&vidchan);
		}
	}

	/* setup channel */
	vidchan.channel = channel;
	if (ioctl(fd,VIDIOCSCHAN,&vidchan) < 0) {
		perror("ioctl(VIDIOCSCHAN)");
		exit(1);
	}

	/* setup video_mmap */
	vidmap.frame = 0;
	vidmap.width = 100;
	vidmap.height = 100;
	vidmap.format = 0;
	if (ioctl(fd,VIDIOCMCAPTURE,&vidmap) < 0) {
		perror("ioctl(VIDIOCMCAPTURE)");
		exit(1);
	}
	close(fd);
}

void dump_vidcap(struct video_capability* vidcap)
{
	printf("Video capabilities:\n===================\n");
	printf("    name: %s\n",vidcap->name);
	printf("    type: ");
	if (vidcap->type & VID_TYPE_CAPTURE) printf("Capture,");
	if (vidcap->type & VID_TYPE_TUNER) printf("Tuner,");
	if (vidcap->type & VID_TYPE_TELETEXT) printf("Teletext,");
	if (vidcap->type & VID_TYPE_OVERLAY) printf("Overlay,");
	if (vidcap->type & VID_TYPE_CHROMAKEY) printf("Chromakey,");
	if (vidcap->type & VID_TYPE_CLIPPING) printf("Clipping,");
	if (vidcap->type & VID_TYPE_FRAMERAM) printf("FrameRam,");
	if (vidcap->type & VID_TYPE_SCALES) printf("Scales,");
	if (vidcap->type & VID_TYPE_MONOCHROME) printf("Monochrome,");
	printf("\nchannels: %d\n",vidcap->channels);
	printf("  audios: %d\n",vidcap->audios);
	printf(" capture: (%d,%d) through (%d,%d)\n",
		vidcap->minwidth,vidcap->minheight,
		vidcap->maxwidth,vidcap->maxheight);
}

void dump_vidchan(struct video_channel* vidchan)
{
	printf("Channel %d capabilities:\n===================\n",vidchan->channel);
	printf("    name: %s\n",vidchan->name);
	printf("  tuners: %d\n",vidchan->tuners);
	printf("   flags: ");
	if (vidchan->type & VIDEO_VC_TUNER) printf("tuner ");
	if (vidchan->type & VIDEO_VC_AUDIO) printf("audio ");
	printf("\n    type: %s\n",vidchan->type==VIDEO_TYPE_TV ? "TV" : vidchan->type==VIDEO_TYPE_CAMERA ? "Camera" : "Unknown");
}
