#include <linux/types.h>
#include <linux/delay.h>
#include <asm/io.h>

#include <linux/version.h>
#include <asm/uaccess.h>

#include "zoran.h"

/* ----------------------------------------------------------------------- */
/* I2C functions							   */
/* ----------------------------------------------------------------------- */

/* software I2C functions */

#define I2C_DELAY   10

static void i2c_setlines(struct i2c_bus *bus,int ctrl,int data)
{
	struct zoran *ztv = (struct zoran*)bus->data;
	unsigned int b = 0;
	if (data) b |= ZORAN_I2C_SCL;
	if (ctrl) b |= ZORAN_I2C_SDA;
	zrwrite(b, ZORAN_I2C);
	udelay(I2C_DELAY);
}

static int i2c_getdataline(struct i2c_bus *bus)
{
	struct zoran *ztv = (struct zoran*)bus->data;
	return zrread(ZORAN_I2C) & ZORAN_I2C_SCL;
}

static
void attach_inform(struct i2c_bus *bus, int id)
{
	struct zoran *ztv = (struct zoran*)bus->data;

	switch (id) {
	 case I2C_DRIVERID_TUNER:
		ztv->have_mux = 1;
		DEBUG(printk(KERN_INFO "zoran: mux attached\n"));
		break;
	 default:
		DEBUG(printk(KERN_INFO "zoran: attach_inform; unknown device id=%d\n",id));
		break;
	}
}

static
void detach_inform(struct i2c_bus *bus, int id)
{
	struct zoran *ztv = (struct zoran*)bus->data;

	switch (id) {
	 case I2C_DRIVERID_TUNER:
		ztv->have_mux = 0;
		DEBUG(printk(KERN_INFO "zoran: mux detached\n"));
		break;
	 default:
		DEBUG(printk(KERN_INFO "zoran: detach_inform; unknown device id=%d\n",id));
		break;
	}
}

struct i2c_bus zoran_i2c_bus_template =
{
	"zoran",
	I2C_BUSID_ZORAN,
	NULL,

	SPIN_LOCK_UNLOCKED,

	attach_inform,
	detach_inform,

	i2c_setlines,
	i2c_getdataline,
	NULL,
	NULL,
};

/* ----------------------------------------------------------------------- */
/* SAA7110 functions							   */
/* ----------------------------------------------------------------------- */
static
int saa7110_selmux(struct i2c_device *device, int *chan)
{
static	const unsigned char modes[9][8] = {
/* mode 1 */	{ 0x00, 0xD9, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16 },
/* mode 2 */	{ 0x00, 0xD8, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16 },
/* mode 3 */	{ 0x00, 0xBA, 0x07, 0x91, 0x03, 0x60, 0xB5, 0x05 },
/* mode 4 */	{ 0x00, 0xB8, 0x07, 0x91, 0x03, 0x60, 0xB5, 0x05 },
/* mode 5 */	{ 0x00, 0x7C, 0x07, 0xD2, 0x83, 0x60, 0xB5, 0x03 },
/* mode 6 */	{ 0x00, 0x78, 0x07, 0xD2, 0x83, 0x60, 0xB5, 0x03 },
/* mode 7 */	{ 0x80, 0x59, 0x17, 0x42, 0xA3, 0x44, 0x75, 0x12 },
/* mode 8 */	{ 0x80, 0x9A, 0x17, 0xB1, 0x13, 0x60, 0xB5, 0x14 },
/* mode 9 */	{ 0x80, 0x3C, 0x27, 0xC1, 0x23, 0x44, 0x75, 0x21 } };
	struct i2c_bus* bus = device->bus;
	const unsigned char* ptr;
	int status;

	/* only 6 channels (9 modes though) */
	if (*chan<0 || *chan>8)
		return -EINVAL;

	ptr = modes[*chan];
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x06,0);		// subaddress
	i2c_sendbyte(bus,ptr[0],0);
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x20,0);		// subaddress
	i2c_sendbyte(bus,ptr[1],0);
	i2c_sendbyte(bus,ptr[2],0);
	i2c_sendbyte(bus,ptr[3],0);
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x2C,0);		// subaddress
	i2c_sendbyte(bus,ptr[4],0);
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x30,0);		// subaddress
	i2c_sendbyte(bus,ptr[5],0);
	i2c_sendbyte(bus,ptr[6],0);
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x21,0);		// subaddress
	i2c_sendbyte(bus,ptr[7],0);
	i2c_stop(bus);

	/* mode changed, start automatic detection */
	mdelay(150);
	status = i2c_read(bus,device->addr|1);
	if ((status & 3) == 0) {
		if (status & 0x20) {
			DEBUG(printk(KERN_DEBUG "zoran: standard=bw60\n"));
			i2c_start(bus);
			i2c_sendbyte(bus,device->addr,0);	// slaveaddress
			i2c_sendbyte(bus,0x06,0);		// subaddress
			i2c_sendbyte(bus,0x80,0);
			i2c_stop(bus);
			i2c_start(bus);
			i2c_sendbyte(bus,device->addr,0);	// slaveaddress
			i2c_sendbyte(bus,0x2E,0);		// subaddress
			i2c_sendbyte(bus,0x81,0);
			i2c_stop(bus);
			*chan = 1;
			return 0;
		}
		DEBUG(printk(KERN_DEBUG "zoran: standard=bw50\n"));
		i2c_start(bus);
		i2c_sendbyte(bus,device->addr,0);	// slaveaddress
		i2c_sendbyte(bus,0x06,0);		// subaddress
		i2c_sendbyte(bus,0x80,0);
		i2c_stop(bus);
		i2c_start(bus);
		i2c_sendbyte(bus,device->addr,0);	// slaveaddress
		i2c_sendbyte(bus,0x2E,0);		// subaddress
		i2c_sendbyte(bus,0x9A,0);
		i2c_stop(bus);
		*chan = 0;
		return 0;
	}

	if (status & 0x20) {
		DEBUG(printk(KERN_DEBUG "zoran: standard=ntsc\n"));
		i2c_start(bus);
		i2c_sendbyte(bus,device->addr,0);	// slaveaddress
		i2c_sendbyte(bus,0x0D,0);		// subaddress
		i2c_sendbyte(bus,0x06,0);
		i2c_stop(bus);
		i2c_start(bus);
		i2c_sendbyte(bus,device->addr,0);	// slaveaddress
		i2c_sendbyte(bus,0x11,0);		// subaddress
		i2c_sendbyte(bus,0x2C,0);
		i2c_stop(bus);
		i2c_start(bus);
		i2c_sendbyte(bus,device->addr,0);	// slaveaddress
		i2c_sendbyte(bus,0x2E,0);		// subaddress
		i2c_sendbyte(bus,0x81,0);
		i2c_stop(bus);
		*chan = 2;
		return 0;
	}

	/* PAL/SECAM */
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x0D,0);		// subaddress
	i2c_sendbyte(bus,0x06,0);
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x11,0);		// subaddress
	i2c_sendbyte(bus,0x59,0);
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x2E,0);		// subaddress
	i2c_sendbyte(bus,0x9A,0);
	i2c_stop(bus);

	mdelay(150);	/* pause 150 ms */

//	if ((status & 0x03) == 0x01) {
	if (0 && (status & 0x03) == 0x01) {
		DEBUG(printk(KERN_DEBUG "zoran: standard=secam\n"));
		i2c_start(bus);
		i2c_sendbyte(bus,device->addr,0);
		i2c_sendbyte(bus,0x0D,0);
		i2c_sendbyte(bus,0x07,0);
		i2c_stop(bus);
		*chan = 4;
		return 0;
	}
	DEBUG(printk(KERN_DEBUG "zoran: standard=pal\n"));
	*chan = 3;
	return 0;
}

static
int saa7110_attach(struct i2c_device *device)
{
static	const unsigned char initseq[] = {
	     0, 0x4C, 0x3C, 0x0D, 0xEF, 0xBD, 0xF0, 0x00, 0x00,
		0xF8, 0xF8, 0x60, 0x60, 0x00, 0x06, 0x18, 0x90,
		0x00, 0x2C, 0x40, 0x46, 0x42, 0x1A, 0xFF, 0xDA,
		0xF0, 0x8B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0xD9, 0x17, 0x40, 0x41, 0x80, 0x41, 0x80, 0x4F,
		0xFE, 0x01, 0xCF, 0x0F, 0x03, 0x01, 0x81, 0x03,
		0x44, 0x75, 0x01, 0x8C, 0x03};
	const unsigned char * const last = initseq+sizeof(initseq);
	const unsigned char* ptr;
	struct i2c_bus* bus = device->bus;

        DEBUG(printk(KERN_DEBUG "saa7110_attach\n"));
	strcpy(device->name, "mux");

	/* setup the chip */
	ptr = initseq;
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	while (ptr != last) {
		if (i2c_sendbyte(bus,*ptr++,0)) {
			i2c_stop(bus);
			DEBUG(printk(KERN_DEBUG "zoran: %s init failed on bus %s, driver %s\n", device->name, bus->name, device->driver->name));
			return -EAGAIN;
		}
	}
	i2c_stop(bus);
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x21,0);		// subaddress
	i2c_sendbyte(bus,0x16,0);
	i2c_stop(bus);

	/* setup and implicit mode 0 select has been performed */
	return 0;
}

static
int saa7110_detach(struct i2c_device *device)
{
	struct i2c_bus* bus = device->bus;
        DEBUG(printk(KERN_DEBUG "saa7110_detach\n"));

	/* stop further output */
	i2c_start(bus);
	i2c_sendbyte(bus,device->addr,0);	// slaveaddress
	i2c_sendbyte(bus,0x0E,0);		// subaddress
	i2c_sendbyte(bus,0x00,0);
	i2c_stop(bus);

        return 0;
}


static
int saa7110_command(struct i2c_device *device, unsigned int cmd, void *arg)
{
	switch (cmd) {
	 case TUNER_SET_MODE:
		return saa7110_selmux(device, arg);
	 default:
		DEBUG(printk(KERN_DEBUG "saa7110_command(%d)??\n",cmd));
		return -EINVAL;
	}
}

struct i2c_driver i2c_driver_mux =
{
        "saa7110",			/* name */
        I2C_DRIVERID_TUNER,		/* in i2c.h */
        0x9C, 0x9D,			/* Addr range */
        saa7110_attach,
        saa7110_detach,
        saa7110_command
};
